from dataclasses import dataclass


@dataclass
class SearchItem:
    title: str
    pageid: int


@dataclass
class Image:
    source: str
    width: int
    height: int

    def __post_init__(self):
        if self.source.find(".gif") != -1:
            raise TypeError("Gif file")

        if self.source.find("revision/latest/scale-to-width-down") != -1:
            revision = self.source.split("/revision")
            rev = revision[1].split("?")[0]
            size = rev.split("/")[-1]

            new_rev = rev.replace(str(size), "10000")

            self.source = self.source.replace(rev, new_rev)



from .drivers import AiohttpDriver
from .exceptions import NotFound

from .methods import ImageController
from .types import Image, SearchItem

from tghtml import TgHTML


WGR_FLAG = (
    "https://upload.wikimedia.org/wikipedia/commons/thumb" +
    "/8/85/Flag_of_Belarus.svg/1000px-Flag_of_Belarus.svg.png"
)

WRW_FLAG = (
    "https://upload.wikimedia.org/wikipedia/commons/thumb" +
    "/5/50/Flag_of_Belarus_%281918%2C_1991%E2%80%931995%29.svg" +
    "/1000px-Flag_of_Belarus_%281918%2C_1991%E2%80%931995%29.svg.png"
)


class Wikipya:
    def __init__(
        self, lang=None, driver=AiohttpDriver, version="1.0",
        url="https://{lang}.wikipedia.org/w/api.php", params={
            "format": "json",
            "action": "query",
            "formatversion": 2
        }, img_blocklist=(), host="", prefix="/w"
    ):
        """ Initialisation instance of Wikipya
        Args:
            lang: Lang of MediaWiki source
            driver: Driver for connection to service
            version: Version of MediaWiki
            url: Template of URL to wiki
            params: Default params for driver
            img_blocklist: List ignored images (lurk patch)
        """

        self.MEDIAWIKI_VERSION = version

        self.img_blocklist = img_blocklist
        self.lang = lang
        self.url = url if lang is None else url.format(
            lang=self.lang
        )
        self.host = host
        self.prefix = prefix

        self.driver = driver(url=self.url,
                             params=params)

    @property
    def params(self):
        return {
            "lang": self.lang,
            "driver": self.driver,
            "version": self.MEDIAWIKI_VERSION,
            "url": self.url,
            "host": self.host
        }

    async def search(self, query, limit=1, prop="size", **kwargs):
        status, data = await self.driver.get(
            list="search",
            srsearch=query,
            srlimit=limit,
            srprop=prop,
            **kwargs
        )

        if len(data.query.search) == 0:
            raise NotFound("Search can't find anything on your request")

        return [SearchItem(
            title=item.__dict__.get("title"),
            pageid=item.__dict__.get("pageid")
        ) for item in data.query.search]

    async def opensearch(self, query, limit=1, **kwargs):
        status, data = await self.driver.get(
            action="opensearch",
            search=query,
            limit=limit,
            **kwargs
        )

        if len(data) == 0:
            raise NotFound("Search can't find anything on your request")

        return data

    async def page(self, query, section=0, prop="text",
                   params={}, **kwargs):
        if query.__class__ == str:
            params = {"page": query}

        elif query.__class__ == int:
            params = {"pageid": query}

        elif query.pageid is not None:
            params = {"pageid": query.pageid}

        else:
            params = {"page": query.title}

        status, data = await self.driver.get(
            action="parse",
            section=section,
            prop=prop,
            redirects="true",
            **params,
            **kwargs
        )

        return WikipyaPage(data.parse, lang=self.lang,
                           img_blocklist=self.img_blocklist,
                           params=self.params)

    async def _page(self, query, exsentences=5, section=0,
                    params={}, **kwargs):
        if exsentences == -1:
            exsentences_json = {}
        else:
            exsentences_json = {"exsentences": exsentences}

        status, data = await self.driver.get(
            prop="extracts",
            titles=query.title,
            formatversion=1,
            **exsentences_json
        )

        result = data.query.pages.__dict__

        if "-1" in result:
            return -1

        query = result[list(result.keys())[-1]]
        query.add({"text": query.extract})

        return WikipyaPage(query, lang=self.lang, 
                           img_blocklist=self.img_blocklist,
                           params=self.params)

    async def image(self, query, pithumbsize=1000, **kwargs):
        return await ImageController(**self.params) \
                        .method(query, pithumbsize,
                                img_blocklist=self.img_blocklist, **kwargs)

    async def getPageName(self, id_):
        status, data = await self.driver.get(
            pageids=id_
        )

        try:
            return data.query.pages[-1].title
        except AttributeError:
            raise NotFound(f"Not found page with this id: {id_}")

    async def get_all(self, query, lurk=False,
                      blocklist=(), **kwargs):
        search = await self.search(query)
        opensearch = await self.opensearch(
            query if lurk else search[0].title
        )

        if lurk:
            try:
                url, title = opensearch[-1][0], opensearch[1][0]
            except:
                url, title = None, search[0]
        else:
            url, title = opensearch[-1], search[0]

        page = await self.page(title)
        page.blockList = blocklist

        try:
            image = await page.image(prefix=self.prefix)
            image = image.source

        except Exception as e:
            print(e)
            image = -1

        if image == WGR_FLAG:
            image = WRW_FLAG

        return page, image, url


class WikipyaPage:
    def __init__(self, parse, params, img_blocklist, lang="en"):
        vars(self).update(parse.__dict__)

        self.blockList = []
        self.lang = lang
        self.params = params      # fastfix for image
        self.img_blocklist = img_blocklist
        self.url = f"https://{lang}.wikipedia.org/w/api.php"

        # fix for lurkmore
        if self.text.__class__.__name__ == "JSONObject":
            self.text = self.text.__dict__["*"]

    @property
    def parsed(self):
        return str(TgHTML(self.text, self.blockList, is_wikipedia=False))

    @property
    def fixed(self):
        namelist = [
            ["Белоруссия", "Беларусь"],
            ["Белоруссии", "Беларуси"],
            ["Беларуссию", "Беларусь"],
            ["Белоруссией", "Беларусью"],
            ["Белоруссиею", "Беларусью"],

            ["Белору́ссия", "Белару́сь"],
            ["Белору́ссии", "Белару́си"],
            ["Белору́ссию", "Белару́сь"],
            ["Белору́ссией", "Белару́сью"],
            ["Белору́ссиею", "Белару́сью"],


            ["на Украин", "в Украин"],
        ]

        for name in namelist:
            text = self.parsed.replace(*name)

        return text

    async def image(self, pithumbsize=1000, **kwargs):
        return await ImageController(**self.params) \
            .method(self.title, pithumbsize, img_blocklist=self.img_blocklist, **kwargs)



from operator import le, lt, eq, gt, ge

from .exceptions import NotFound, NotSupported
from .types import Image

from bs4 import BeautifulSoup


OPERATORS = {"<=": le, "<": lt, "==": eq, ">": gt, ">=": ge}


class BaseContoller:
    def __init__(self, lang, driver, version, url, host):
        self.lang = lang
        self.driver = driver
        self.version = version
        self.url = url
        self.host = host

        self.thumb = url.replace("api.php", "thumb.php")

        for method in self.METHODS:
            if check_support(version, self.METHODS[method]):
                self.method = getattr(self, method)
                return

        raise NotSupported(f"Not found methods that support MediaWiki v{version}")


def check_support(version, requirement):
    for i in (1, 2):
        try:
            depend = float(requirement[i:])
            operator = requirement[:i]

            break
        except ValueError:
            pass

    return OPERATORS[operator](float(version), depend)


class ImageController(BaseContoller):
    METHODS = {
        "image": ">=1.25",
        "parse_images": ">=1.0"
    }

    async def image(self, titles, pithumbsize=1000,
                    piprop="thumbnail", img_blocklist=(), prefix="w",
                    **kwargs):
        status, data = await self.driver.get(
            titles=titles,
            prop="pageimages",
            pilicense="any",
            pithumbsize=pithumbsize,
            piprop=piprop,
            **kwargs
        )

        try:
            image = data.query.pages[-1]
            thumb = image.thumbnail

            return Image(**thumb.__dict__)

        except AttributeError:
            raise NotFound("Not found image")

    async def parse_images(self, titles, pithumbsize=1000,
                           piprop="thumbnail", img_blocklist=(),
                           prefix="w", **kwargs):
        status, data = await self.driver.get(
            action="parse",
            page=titles,
            prop="images",
            redirects="true",
            section=0
        )

        all_images = data.parse.images

        if len(all_images) == 0:
            raise NotFound("Not found images")
        else:
            images = []

            for image in all_images:
                if image not in img_blocklist:
                    try:
                        images.append(await self.get_image(
                                          image,
                                          prefix=prefix))
                        return images[0]
                    except Exception as e:
                        print(e)

    async def get_image(self, name, prefix="w"):
        url = self.url.lower() \
                      .replace('/wiki/api.php', prefix) \
                      .replace("/w/api.php", prefix) \
                      .replace("/api.php", prefix) \

        status, data, url = await self.driver.get_html(
            f"{url}/File:{name}"
        )

        if status != 200:
            return 404

        soup = BeautifulSoup(data, 'lxml')

        file = soup.find("div", id="file")
        thumbnails = soup.find_all("img", {"class": "pi-image-thumbnail"})
        thumbs = soup.find_all("figure", {"class": "thumb"})
        navigation = soup.find_all("nav", {"class": "pi-navigation"})

        if file is not None:
            return Image(
                source="https:" + self.host + soup.find("div", id="file").a.img["src"],
                width=0, height=0
            )

        elif len(thumbnails) != 0:
            image = thumbnails[0]

            return Image(
                source=image["srcset"].split()[-2],
                width=image["width"], height=image["height"]
            )

        elif len(navigation) != 0:
            image = navigation[0].a.img

        elif len(thumbs) != 0:
            image = thumbs[0].a.img

        else:
            raise NotFound("Not found image")

        return Image(
            source=image["src"],
            width=image["width"], height=image["height"]
        )



class NotFound(Exception):
    def __init__(self, text):
        pass


class NotSupported(Exception):
    def __init__(self, text):
        pass


class ParseError(Exception):
    def __init__(self, text):
        pass



import aiohttp

import time
import json

from .exceptions import ParseError


class JSONObject:
    """JSON => Class"""
    def __init__(self, dict):
        self.add(dict)

    def add(self, dict):
        vars(self).update(dict)


class BaseDriver:
    def __init__(self, url="example.com", timeout=5, params=()):
        self.timeout = timeout
        self.params = params
        self.url = url

    async def get(self, url, params=(), timeout=None):
        raise NotImplementedError


class AiohttpDriver(BaseDriver):
    async def get(self, url=None, timeout=None, debug=False, **params):
        if len(list(params.keys())) == 0:
            params = None
        else:
            params = {**self.params, **params}

        async with aiohttp.ClientSession() as session:
            start = time.time()

            async with session.get(
                url or self.url, params=params,
                timeout=timeout or self.timeout
            ) as response:
                text = await response.text()

                if debug:
                    print(response.url)
                    print(time.time() - start)

                js = json.loads(text, object_hook=JSONObject)

                if isinstance(js, list):
                    pass

                elif js.__dict__.get("error") is not None:
                    raise ParseError(f"{js.error.code}: {js.error.info}")

                return response.status, js

    async def get_html(self, url=None, timeout=None, debug=False, **params):
        if len(list(params.keys())) == 0:
            params = None
        else:
            params = {**self.params, **params}

        async with aiohttp.ClientSession() as session:
            start = time.time()

            async with session.get(
                url or self.url, params=params,
                timeout=timeout or self.timeout
            ) as response:
                text = await response.text()

                if debug:
                    print(response.url)
                    print(time.time() - start)

                return response.status, text, response.url



from setuptools import setup
from wikipya import __version__

with open("README.md", encoding="utf-8") as readme:
    long_description = readme.read()

setup(
    name="wikipya",
    version=__version__,
    author="Daniel Zakharov",
    author_email="daniel734@bk.ru",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="api wikipedia python",
    url="https://github.com/jDan735/wikipya",
    license="MIT",
    packages=["wikipya"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires=">=3",
    install_requires=[
        "beautifulsoup4",
        "tghtml"
    ],
    test_suite="tests"
)
